#!/bin/bash
#
# RoksNet Portal web application package
#
# Aktors 2016

xrd_prefix=/usr/xtee
tomcat_home=/var/lib/tomcat7
app_name=ROOT
host=127.0.0.1
port=5432
db_name=misp2db
username=misp2
username_pass=
PIN2=00000
enable_encrypt=y
enable_signing=y
config_mobile_id=n
config_https=n
m_id_service_name=Teenus
install_default=upgrade
email_host=smtp.domain.ee
email_sender=info@domain.ee
apache2=/etc/apache2
# 'y' if portal is configured in international mode, 'n' if not; value could be replaced before package generation
configure_international=y
# 'y' to skip estonian portal related prompt questions, 'n' to include them; value could be replaced before package generation
skip_estonian=y

#####################
# Declare functions #
#####################
##
# @return success code (0) if RoksNet Portal deployment directory with conf files exist
#         failure code (1) if RoksNet Portal deployment directory or conf files do not exist
##
function misp2_deployed {
	deploy_dir=$tomcat_home/webapps/$app_name
	classes_dir=$deploy_dir/WEB-INF/classes
	meta_inf_dir=$deploy_dir/META-INF

	# Check whether webapp files exist that indicate deployment in Tomcat
	if	[[ -d $classes_dir 			]] &&
		[[ -f $classes_dir/config.cfg 		]] && 
		[[ -f $classes_dir/orgportal-conf.cfg	]] && 
		[[ -f $classes_dir/uniportal-conf.cfg	]] && 
		[[ -f $classes_dir/log4j.properties	]] && 
		[[ -d $meta_inf_dir 			]] && 
		[[ -f $meta_inf_dir/context.xml 	]]
	then
		# Else webapp has been deployed
		return 0
	else
		# If not all deployment directory files exist, webapp has not yet deployed
		return 1
	fi
}

##
# Wait until RoksNet Portal webapp has been deployed and echo out waiting status
##
function wait_for_misp2_deployment {
	start_time=$SECONDS
	time_spent=""
	while	! misp2_deployed
	do
		time_spent=$(($SECONDS - $start_time))
		echo -ne "...Waiting for RoksNet Portal webapp deployment... ($time_spent s)"\\r
		sleep 0.5
	done
	sleep 1
	# Add another newline if previous entry was a line update
	[ "$time_spent" != "" ] && echo
	echo "...RoksNet Portal webapp deployment done..."
}

##
# @return success code (0) if RoksNet Portal deployment directory and WAR does not exist
#         failure code (1) if RoksNet Portal deployment directory or WAR exists
##
function misp2_undeployed {
	deploy_dir=$tomcat_home/webapps/$app_name
	war_full_path=$deploy_dir.war
	# Check whether webapp Tomcat deployment directory or the corresponding WAR file exist
	if	[[ -d $deploy_dir 			]] ||
		[[ -f $war_full_path			]]
	then
		# WAR or deployment directory still exists, webapp has not yet been undeployed
		return 1
	else
		# Neither WAR nor deployment directory exist, webapp has totally undeployed
		return 0
	fi
}

##
# Wait until RoksNet Portal webapp has been undeployed and echo out waiting status
##
function wait_for_misp2_undeployment {
	start_time=$SECONDS
	time_spent=""
	while	! misp2_undeployed
	do
		time_spent=$(($SECONDS - $start_time))
		echo -ne "...Waiting for RoksNet Portal webapp undeployment... ($time_spent s)"\\r
		sleep 0.5
	done
	sleep 1
	# Add another newline if previous entry was a line update
	[ "$time_spent" != "" ] && echo
	echo "...RoksNet Portal webapp undeployment done..."
}


##############################################
# Begin RoksNet Portal package installation
##############################################

# Ask tomcat location
if [ ! -d $tomcat_home/webapps ]
then
	echo -n "Please provide Apache Tomcat working directory [default: $tomcat_home]: " 
	read user_tomcat < /dev/tty
	if [ "$user_tomcat" == "" ]
	then
			user_tomcat=$tomcat_home
	fi
	tomcat_home=$user_tomcat
fi

if [ ! -d $tomcat_home/webapps ]
then
	echo "$tomcat_home/webapps is not found"
	exit 1
fi

# If misp2 deploy directory exists then upgrade else install
if [ -d $tomcat_home/webapps/$app_name ]
then
	echo "Found RoksNet Portal deploy directory so upgrading.."
	install_default=upgrade
else
	echo "Did not find RoksNet Portal deploy directory so installing new.."
	install_default=install
fi


xroad_instances="roksnet-dev,roksnet"
xroad_member_classes="COM,NGO,GOV,NEE"

if [ "$install_default" == "upgrade" ]
then
	echo " === Upgrading RoksNet Portal application  ==="
	echo " "
	
### copy war file to the tomcat webapps directory
### backuping configuration
	echo " === Backing up configuration === "
	cp $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg /tmp/config.cfg.bkp
	cp $tomcat_home/webapps/$app_name/WEB-INF/classes/orgportal-conf.cfg /tmp/orgportal-conf.cfg.bkp
	cp $tomcat_home/webapps/$app_name/WEB-INF/classes/uniportal-conf.cfg /tmp/uniportal-conf.cfg.bkp
	cp $tomcat_home/webapps/$app_name/WEB-INF/classes/log4j.properties /tmp/log4j.properties.bkp
	
	#Synchronize existing config properties with default ones
	java -Xmx1024M -jar $xrd_prefix/app/propertySynchronizer.jar -s $xrd_prefix/app/config.orig.cfg -t /tmp/config.cfg.bkp -r /tmp/config.cfg.bkp -e ISO-8859-1
	if [ $? -ne 0 ]
	then
		echo "Config properties synchronization has failed"
		exit 1
	fi
	
	#Synchronize existing log4j properties with default ones
	java -Xmx1024M -jar $xrd_prefix/app/propertySynchronizer.jar -s $xrd_prefix/app/log4j.properties -t /tmp/log4j.properties.bkp -r /tmp/log4j.properties.bkp -e ISO-8859-1
	if [ $? -ne 0 ]
	then
		echo "log4j properties synchronization has failed"
		exit 1
	fi
	
# check if localhost:8080 exists and rewrite it to localhost (since v. 1.20 Tomcat port 8080 in closed)
	sed 's/localhost:8080/localhost/g' -i /tmp/config.cfg.bkp
# remove ^M from config file
	sed -i s/\\r//g /tmp/config.cfg.bkp
# rewrite context file
	perl -pi -e "s/APP_NAME/$app_name/g" $xrd_prefix/app/context.orig.xml
	perl -pi -e "s/APP_NAME/$app_name/g" $xrd_prefix/app/admintool.sh

	echo " === Undeploying previous version of RoksNet Portal web application === "
	rm -rf $tomcat_home/webapps/$app_name*
	wait_for_misp2_undeployment
	echo " === Deploying new version of RoksNet Portal web application === "
	cp $xrd_prefix/app/*.war $tomcat_home/webapps/$app_name.war
	wait_for_misp2_deployment
	
	echo " === Restoring configuration === "
### restoring configuration
	if [ ! -d $tomcat_home/webapps/$app_name/WEB-INF/classes -o ! -d $tomcat_home/webapps/$app_name/META-INF ]
	then
		echo "WARNING! Previous configuration could not be restored. Either Tomcat was not running or deployment of the application did not finish in time. When installation is complete copy the files /tmp/config.cfg.bkp (to $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg), /tmp/log4j.properties.bkp (to $tomcat_home/webapps/$app_name/WEB-INF/classes/log4j.properties) and $xrd_prefix/app/context.orig.xml (to $tomcat_home/webapps/$app_name/META-INF/context.xml) manually."
	else
		cp /tmp/config.cfg.bkp $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
		cp /tmp/orgportal-conf.cfg.bkp $tomcat_home/webapps/$app_name/WEB-INF/classes/orgportal-conf.cfg
		cp /tmp/uniportal-conf.cfg.bkp $tomcat_home/webapps/$app_name/WEB-INF/classes/uniportal-conf.cfg
		cp /tmp/log4j.properties.bkp $tomcat_home/webapps/$app_name/WEB-INF/classes/log4j.properties 
		#cp /tmp/context.xml.bkp $tomcat_home/webapps/$app_name/META-INF/context.xml - disabled since 1.24 because we want to make sure that context is correct
		cp $xrd_prefix/app/context.orig.xml $tomcat_home/webapps/$app_name/META-INF/context.xml
	fi
	
### replacing new key values in configuration
	if grep -q "languages = en" $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
	then
		configure_international="y"
		echo "Updating international version"
	else
		configure_international="n"
		echo "Updating Estonian version"
	fi
	if grep -q 'XROAD_INSTANCES' $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
	then
		# Prompt for user input if configure_international=y and international_xroad_instances variable is set
		if [ "$configure_international" == "y" ] && [ -n "${international_xroad_instances+x}" ]
		then
			xroad_instances=$international_xroad_instances
			echo -n "Please provide RoksNet v6 instances (comma separated list)? [default: $xroad_instances]"
			read user_xroad_instances < /dev/tty
			if [ "$user_xroad_instances" != "" ]
			then
				xroad_instances=$user_xroad_instances
			fi
		fi
		echo "Updating RoksNet instances $xroad_instances"
		perl -pi -e "s/XROAD_INSTANCES/$xroad_instances/" $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
	fi
	
	if grep -q 'XROAD_MEMBER_CLASSES' $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
	then
		# Prompt for user input if configure_international=y and international_member_classes variable is set
		if [ "$configure_international" == "y" ] && [ -n "${international_member_classes+x}" ]
		then
			xroad_member_classes=$international_member_classes
			echo -n "Please provide RoksNet v6 member classes (comma separated list)? [default: $xroad_member_classes]"
			read user_xroad_member_classes < /dev/tty
			if [ "$user_xroad_member_classes" != "" ]
			then
				xroad_member_classes=$user_xroad_member_classes
			fi
		fi
		echo "Updating RoksNet member classes $xroad_member_classes"
		perl -pi -e "s/XROAD_MEMBER_CLASSES/$xroad_member_classes/" $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
	fi
fi


if [ "$install_default" == "install" ]
then

	echo " === Deploying RoksNet Portal web application === "
	echo " "
	### copy war file to the tomcat webapps directory
	cp $xrd_prefix/app/*.war $tomcat_home/webapps/$app_name.war
		
	# Only prompt when estonian portal related questions are not skipped
	if [ "$skip_estonian" != "y" ]
	then
		echo -n "Do you want to configure as international version (if no, then will be configured as estonian version)? [y/n] [default: n]: "
		read configure_international < /dev/tty
	fi

	# Override original config properties with international config properties
	# Synchronize international conf with original, if application is configured as international version
	if [ "$configure_international" == "y" ]
	then
		java -Xmx1024M -jar $xrd_prefix/app/propertySynchronizer.jar -s $xrd_prefix/app/config.orig.cfg -t $xrd_prefix/app/config.origForInternational.cfg -r $xrd_prefix/app/config.orig.cfg -e ISO-8859-1
		if [ $? -ne 0 ]
		then
			echo "Original and international config synchronization has failed"
			exit 1
		fi
	fi

	### database config

	echo -n "Please provide database host IP to be used [default: $host]: "
	read user_host < /dev/tty
	if [ "$user_host" == "" ]
	then
		user_host=$host
	fi
	host=$user_host

	echo -n "Please provide database port to be used [default: $port]: "
	read user_port < /dev/tty
	if [ "$user_port" == "" ]
	then
		user_port=$port
	fi
	port=$user_port


	echo -n "Please provide database name to be used [default: $db_name]: "
	read user_db < /dev/tty
	if [ "$user_db" == "" ]
	then
		user_db=$db_name
	fi
	db_name=$user_db


	echo -n "Please provide username to be communicating with database [default: $username]: "
	read user_username < /dev/tty
	if [ "$user_username" == "" ]
	then
		user_username=$username
	fi
	username=$user_username

	read -s -p "Please enter username password: [default: $username_pass]: " username_password
	if [ "$username_password" == "" ]
	then
		username_password=$username_pass
	fi
	username_pass=$username_password

	echo ""
	if [ "$skip_estonian" != "y" ]
	then
		echo -n "Do you want to configure signing and encrypting of Estonian ID-card certificates (y/n)? [default: y] "
		read config_id < /dev/tty
		if [ "$config_id" == "" ]
		then
			config_id="y"
		fi
	
		if [ `echo $config_id | grep -i y ` ]
		then
		### PIN2
			read -s -p "Please enter PIN2: [default: $PIN2]: " user_PIN2
			if [ "$user_PIN2" == "" ]
			then
				user_PIN2=$PIN2
			fi
			PIN2=$user_PIN2
			
			echo ""
			
		### encrypting
			echo -n "Turn on using of encrypting: [y/n] [default: $enable_encrypt]: "
			read user_encrypt < /dev/tty
			if [ "$user_encrypt" == "" ]
			then
				user_encrypt=$enable_encrypt
			fi
			enable_encrypt=$user_encrypt
	
		### signing	
			echo -n "Turn on using of digital signing: [y/n] [default: $enable_signing]: "
			read  user_signing < /dev/tty
			if [ "$user_signing" == "" ]
			then
				user_signing=$enable_signing
			fi
			enable_signing=$user_signing
		else
			enable_signing=n
			enable_encrypt=n
		fi
		### configure Mobile-ID 
		###

		echo -n "Do you want to enable authentication with Mobile-ID? [y/n] [default: $config_mobile_id] "
		read config_mobile_id < /dev/tty
		if [ "$config_mobile_id" == "" ]
		then
				config_mobile_id="n"
		fi


		if [ `echo $config_mobile_id | grep -i y ` ]
		then
			echo -n "Please provide Mobile-ID service name [default: $m_id_service_name]: "
				read user_m_id_service_name < /dev/tty
				if [ "$user_m_id_service_name" == "" ]
				then
						user_m_id_service_name=$m_id_service_name
				fi
				m_id_service_name=$user_m_id_service_name

		fi
	fi


	### configure mail servers
	##
	echo -n "Please provide SMTP host address [default: $email_host]: "
			read user_email_host < /dev/tty
			if [ "$user_email_host" == "" ]
			then
					user_email_host=$email_host
			fi
			email_host=$user_email_host

	### sender address
			echo -n "Please provide server email address: [default: $email_sender]: "
			read  user_email_sender < /dev/tty
			if [ "$user_email_sender" == "" ]
			then
					user_email_sender=$email_sender
			fi
			email_sender=$user_email_sender
	email_sender=$(echo $email_sender | sed 's/\@/\\@/g')

	# Prompt for user input if configure_international=y and international_xroad_instances variable is set
	if [ "$configure_international" == "y" ] && [ -n "${international_xroad_instances+x}" ]
	then
		xroad_instances=$international_xroad_instances
		echo -n "Please provide RoksNet v6 instances (comma separated list)? [default: $xroad_instances]"
		read user_xroad_instances < /dev/tty
		if [ "$user_xroad_instances" != "" ]
		then
			xroad_instances=$user_xroad_instances
		fi
	fi
	
	# Prompt for user input if configure_international=y and international_member_classes variable is set
	if [ "$configure_international" == "y" ] && [ -n "${international_member_classes+x}" ]
	then
		xroad_member_classes=$international_member_classes
		echo -n "Please provide RoksNet v6 member classes (comma separated list)? [default: $xroad_member_classes]"
		read user_xroad_member_classes < /dev/tty
		if [ "$user_xroad_member_classes" != "" ]
		then
			xroad_member_classes=$user_xroad_member_classes
		fi
	fi

	### updating configuration files using perl replace
	### config.cfg
	perl -pi -e "s/MISP2DBHOST/$host/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/MISP2DBPORT/$port/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/MISP2DBNAME/$db_name/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/MISP2DBUSERNAME/$username/" $xrd_prefix/app/config.orig.cfg
	
	# replace '/' with '\/' to create regex replacement string where slashes are quoted
	username_pass="${username_pass//\//\\\/}"
	perl -pi -e "s/MISP2DBPASSWORD/$username_pass/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/ID_PIN2/$PIN2/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/EMAILHOST/$email_host/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/EMAILSENDER/$email_sender/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/XROAD_INSTANCES/$xroad_instances/" $xrd_prefix/app/config.orig.cfg
	perl -pi -e "s/XROAD_MEMBER_CLASSES/$xroad_member_classes/" $xrd_prefix/app/config.orig.cfg

	if [ `echo $enable_encrypt | grep -i y ` ]
	then
		perl -pi -e "s/ALLOW_ENCRYPT/true/" $xrd_prefix/app/config.orig.cfg
	else
		perl -pi -e "s/ALLOW_ENCRYPT/false/" $xrd_prefix/app/config.orig.cfg
	fi

	if [ `echo $enable_signing | grep -i y ` ]
	then
		perl -pi -e "s/ALLOW_SIGNING/true/" $xrd_prefix/app/config.orig.cfg
	else
		perl -pi -e "s/ALLOW_SIGNING/false/" $xrd_prefix/app/config.orig.cfg
	fi

	if [ `echo $config_mobile_id | grep -i y ` ]
	then
		perl -pi -e "s/M_ID_SERVICE_NAME/$m_id_service_name/" $xrd_prefix/app/config.orig.cfg
	fi
	sed -i s/\\r//g $xrd_prefix/app/config.orig.cfg

	### META-INF/context.xml config
	perl -pi -e "s/APP_NAME/$app_name/g" $xrd_prefix/app/context.orig.xml
	perl -pi -e "s/APP_NAME/$app_name/g" $xrd_prefix/app/admintool.sh

	wait_for_misp2_deployment

	echo "Copying configuration files..."
	cp $xrd_prefix/app/config.orig.cfg $tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg
	exit1=$?
	cp $xrd_prefix/app/context.orig.xml $tomcat_home/webapps/$app_name/META-INF/context.xml
	exit2=$?
	cp $xrd_prefix/app/log4j.properties $tomcat_home/webapps/$app_name/WEB-INF/classes/
	exit3=$?
	echo "Copying certificates if they exist..."
	if [ -f $apache2/ssl/MISP2_CA_cert.pem ]
		then cp $apache2/ssl/MISP2_CA_cert.pem $tomcat_home/webapps/$app_name/WEB-INF/classes/certs/MISP2_CA_cert.pem
		echo "Copying certificates 1"
	fi
	if [ -f $apache2/ssl/MISP2_CA_key.pem ]
		then cp $apache2/ssl/MISP2_CA_key.pem $tomcat_home/webapps/$app_name/WEB-INF/classes/certs/MISP2_CA_key.pem
	fi
	if [ -f $apache2/ssl/MISP2_CA_key.der ]
		then cp $apache2/ssl/MISP2_CA_key.der $tomcat_home/webapps/$app_name/WEB-INF/classes/certs/MISP2_CA_key.der
	fi
	if [ $exit1 -ne 0 -o $exit2 -ne 0 -o $exit3 -ne 0 ]
	then
		echo "Cannot copy files. Maybe they haven't yet been deployed by Tomcat. Please make sure that Tomcat is running and rerun the installation. Exit codes: $exit1 $exit2 $exit3"
		exit 1
	else
		echo "Configuration files created"
		echo -n "Do you want to add new administrator account? [y/n] [default: y]"
		read admin_add < /dev/tty
		if [ "$admin_add" == "" ]
		then
				admin_add="y"
		fi
		if [ `echo $admin_add | grep -i y ` ]
		then
			echo "Adding administrator account: "
			$xrd_prefix/app/admintool.sh -add
			$xrd_prefix/app/configure_admin_interface_ip.sh change
		fi
	fi
	
	echo -n "Do you want to enable HTTPS connection between RoksNet Portal application and security server? [y/n] [default: n] "
	read config_https < /dev/tty
	if [ "$config_https" == "" ]
	then
		config_https="n"
	fi
	if [ `echo $config_https | grep -i y ` ]
	then
		$xrd_prefix/app/create_https_certs_security_server.sh omitrestart
	fi

fi

#Remove cached jsp-s, because for some reason Tomcat does not recompile jsp-s currently. After this deletion however, tomcat will compile jsp-s
rm -f -r /var/cache/tomcat7/Catalina/localhost/$app_name/org/apache/jsp

echo "Restarting Tomcat..."
if [ ! -f /etc/init.d/tomcat7 ]
then 
	echo "Shutdown Tomcat..."
	$tomcat_home/bin/shutdown.sh
	echo "Tomcat starting up..."
	$tomcat_home/bin/startup.sh
else
	/etc/init.d/tomcat7 restart
fi



echo "Successfully installed application $app_name"
echo "You can change the configuration of application later by editing this file: "
echo "$tomcat_home/webapps/$app_name/WEB-INF/classes/config.cfg"
